"""
Rational RBF approximation of the 1d advection equation
  u_t + u_x = 0  on the interval [-1,1] with a Dirichlet BC u(-1) = 1
  
  
reference: S. A. Sarra and Y. Bai. A rational radial basis function method for accurately 
           resolving discontinuities and steep gradients. Applied Numerical Mathematics 130, 
           p. 131-142, 2018.  DOI: 10.1016/j.apnum.2018.04.001

usage: run adv1dRRBF.py

"""

import numpy as np
import matplotlib.pyplot as plt
from numpy.linalg import norm
import AnimatePlot as ap
from iqx import *
import rbfRational as rat
import rbfCenters as rc
import rbfMisc as rm

phi = iqx()
s = 6.0
N = 100
k = 0.005
P = 10           # plot every P time step
finalT = 0.75
x = rc.xKT(N,-1,1,0.999)
r = phi.distanceMatrix1d(x)
B = phi.rbf(r,s)
H, H2 = phi.D1(r,s,r), phi.D2(r,s,r)
L, Bi = rat.rbfRationalCoreMultipleSetUp(B)

def F(u,t):
	u[0] = 1.0
	ra1, _ = rat.diff1dMultiple(u,B,Bi,L,H,H2)
	return -ra1

def exact(x,t):
	x2 = x - t
	conditions = [x2 <= -0.75,  x2 > -0.75]
	functions = [lambda x: 1.0, lambda x: -1.0]
	return np.piecewise(x2, conditions, functions)
	
def makePlot(): 
	plt.plot(x, u, 'b')
	plt.plot(x, exact(x,t),'r--')
	plt.ylim([-1.1,1.1])
	
	
u = exact(x,0)
n, t = 0, 0
while n*k < finalT:
	u = rm.rk4(u,n*k,k,F)
	n += 1
	t += k
	u[0] = 1.0
	if n%P==0: ap.animate(makePlot)
	
print('error: {:1.2e}'.format( norm( u - exact(x,finalT), np.inf)))




