"""
Uses RBF partion of unity (PU) differentiation to approximate 
L1 = u_x + u_y and L2 = u_{xx} + u_{yy} on a cicular domain

The NSPD method is used to select the shape parameter on each patch
and MDI regularization is used in forming the local differentiation matrices
or each patch

usage: run differentiationPU.py

"""

import matplotlib.pyplot as plt
from numpy.linalg import norm
from numpy import zeros
import rbfCenters as rc
from rbfClassX import rbfx
from iqx import *
import warnings                     # ignore warnings of ill-conditioning
warnings.filterwarnings('ignore')

SHAPESPD = True           # either constant shape or NSPD selection
N = 1500                  # number of centers
xc, yc = rc.circleCenters(N,ch=3,center=(0.5,0.5))
N, X = len(xc), list( zip(xc,yc) )

from F2b import *   # derivative of this function being approximated
fn = F2b()
f = fn.F(xc,yc)
fDiv, fLap = fn.div(xc,yc), fn.L(xc,yc)  # exact derivatives

phi = iqx()             # inverse quadratic RBF

import rbfPU as PU
from W32 import *
phiC = W32()            # Wendland compactly supported RBF

s = 1.8                 # same shape of each patch if SHAPESPD = False
radius, x, y, st = PU.cover2d(xc,yc,5,0.75,0)
n = len(st)                                     # number of patches
w = PU.patchWeights(st,x,y,xc,yc,radius,phiC)

# different shape of each patch via SPD shape parameter selection
if SHAPESPD: sp = PU.patchShape(X,st,phi,mu=0,s0=0.5,ds=0.01)  

D1x, D1y, D2x, D2y = [], [], [], []
for k in range(n):
	st0 = st[k]
	r, rx, ry = rbfx.distanceMatrix2d(xc[st0],yc[st0])

	if SHAPESPD:
		Hx, Hy = phi.D1(r,sp[k],rx), phi.D1(r,sp[k],ry)
		B, Hxx, Hyy = phi.rbf(r,sp[k]), phi.D2(r,sp[k],rx), phi.D2(r,sp[k],ry)
	else:
		Hx, Hy = phi.D1(r,s,rx), phi.D1(r,s,ry)
		B, Hxx, Hyy = phi.rbf(r,s), phi.D2(r,s,rx), phi.D2(r,s,ry)
	
	D1x.append( phi.dm( B, Hx, 5e-15, False ) )  # local differentiation matrices
	D1y.append( phi.dm( B, Hy, 5e-15, False ) )
	D2x.append( phi.dm( B, Hxx, 5e-15, False ) )
	D2y.append( phi.dm( B, Hyy, 5e-15, False ) )
	
#      2 ways to evaluate the derivatives

# 1) loop base approach to derivative approximation

w, wp, wpp, wpy, wppy = PU.patchWeightsD(st,x,y,xc,yc,radius,phiC)
fx, fy, fxx, fyy = PU.evaluateDerivates(f,st,D1x,D1y,D2x,D2y,w,wp,wpp,wpy,wppy,True)


n1 = zeros((N,))
n2 = zeros((N,))
n3 = zeros((N,))
for k in range(n):
	st0, w0, wp0, wpp0, wpy0, wppy0 = st[k], w[k], wp[k], wpp[k], wpy[k], wppy[k]
	n1[st0] += ( diag(wp0) + diag(wpy0) )@f[st0]         # neglected (theorectically zeros) term for L1
	n2[st0] += ( diag(wpp0) + diag(wppy0) )@f[st0]        # neglected (theorectically zeros) term for L2
	n3[st0] += 2*diag(wp0)@(D1x[k]@f[st0]) + 2*diag(wpy0)@(D1y[k]@f[st0])    # neglected second derivative term which is the difference between the 
	                                                                         #  standard and modified PU methods
	
print('standard PU divergence error {:1.15e}'.format( norm( (fx + fy) - fDiv, inf )))
print('standard PU Laplacian error: {:1.15e}'.format( norm( (fxx + fyy) - fLap, inf )))
print(' ')

# 2) matrix-vector approach to derivative approximation
d1x, d2x = PU.globalDM(N,st,w,wp,D1x,D2x)
d1y, d2y = PU.globalDM(N,st,w,wpy,D1y,D2y)

D1 = d1x + d1y
D2 = d2x + d2y
 
fadAlt, falAlt = D1@f, D2@f
	
print('standard PU divergence error, DM: {:1.15e}'.format( norm( fadAlt - fDiv, inf )))
print('standard PU Laplacian error, DM: {:1.15e}'.format( norm( falAlt - fLap, inf )))
print(' ')


sm = 0
for k in range(n):
    sm += len(st[k])
print('average number of centers per patch: {:d}'.format(round(sm/n)))

plt.close('all')
plt.semilogy(abs(n1),'b.')
plt.figure()
plt.semilogy(abs(n2),'g.')
plt.figure()
plt.semilogy(abs(n3),'c.')
plt.show()


