"""
Solves a 2d diffusion equation with a forcing term

u_t = u_{xx} + u_{yy} + 4k^2(x^2 + y^2) \cos(k \left[ t - x^2 -y^2 \right]) - 5 k \sin(k \left[t - x^2 -y^2 \right])

in a circular domain with a local RBF method.  The initial conditions
and Dirichlet boundary conditions are taken from the exact solution

u(x,y,t) = 1 + \cos(k \left[t - x^2 - y^2 \right])

usage: run heatCircleLocal.py

see also: heatCircleLocalsEws.py which calculates the eigenvalues of the
          discretized equation with zero Dirichlet BCs applied.
          
"""

import matplotlib.pyplot as plt
from numpy import arange, zeros_like, sin, cos, inf
from numpy.linalg import norm
import rbfLocal as LR
import matplotlib.tri as tri
import AnimatePlot as ap
import rbfCenters as rc
import rbfMisc as rm

from rbfClassX import rbfx
from gax import *

import warnings
warnings.filterwarnings('ignore')
plt.close('all')

phi = gax()

MDI = True   
FILTER = False
LAP = True
SAFE = False   # LU or Cholesky

mu, nuG, nuL = 0, 0, 0
order = 2   # Laplacian order

def LO(r,x,y,c,phi): return phi.L(r,c)

# --------------- centers -------------------------------------

N, Nb = 6000, 350
bpi = arange(Nb)
xi, yi = rc.circleCenters(N,cluster=False,ch=3,R=1,plt=False)
x, y = rc.circleBndCenters(xi,yi,1,Nb,tol=0.0125,plt=True)
x += 1
y += 1

triang = tri.Triangulation(x, y)
t, dt, finalT = 0, 0.00004, 0.5
showPlot = 1000

print('boundary centers: {:d}'.format(Nb))
print('   total centers: {:d}'.format(len(x)))

if MDI:           # MDI regularization parameter
	mu = 5e-15           
if FILTER:        # hyperviscosity - inverse system matrix filter
	nuG = 1e-10     
if LAP:           # hyperviscosity - Laplacian power
	nuL = 1e-13

print('            MDI mu: {:1.2e}'.format(mu))
print(' inverse system nu: {:1.2e}'.format(nuG))
print('Laplacian power nu: {:1.2e}'.format(nuL))

# ------------ local setup -----------------------

ns = 25
X = list( zip(x,y) )
st = LR.stencils(X,ns)  

s0 = 2.4
ds = 0.01
s = LR.shapes(X,st,phi,0,s0,ds,verbose=False)
D = LR.weights(X,st,phi,s,LO,mu,nuG,nuL,2,order)
D[bpi,:] = 0

# ----------------------------------------------------------------

K = 2
def exactSolution(x,y,t): return 1.0 + cos(K*t - K*x**2 - K*y**2)

U = exactSolution(x,y,t)
u = zeros_like(U)

def F(v,t):
    v[bpi] = exactSolution(x[bpi],y[bpi],t)
    return D@v + 4*K**2*(x**2 + y**2)*cos(K*t - K*x**2 - K*y**2) - 5*K*sin(K*t - K*x**2 - K*y**2)
    
def makePlot(): 
	plt.tricontour(x, y, u, 15, linewidths=1.5, colors='k')
	plt.tricontourf(x, y, u, 15, cmap=plt.cm.jet)


n = 0
while t<finalT:
    u = rm.rk4(U,t,dt,F)
    u[bpi] = exactSolution(x[bpi],y[bpi],t)
    if n % showPlot == 0:
        ap.animate(makePlot)
    t += dt
    n += 1
    U = u.copy()

plt.colorbar()
exact = exactSolution(x,y,t)
print('max error: {:1.2e}'.format(norm( u-exact,inf)/max(abs(u))))
