"""
Examines eigenvalue (ew) stability of the local RBF method for the 2d diffusion 
equation with zero Dirichlet eigenvalues on a circular domain.

Three options are available for improving ew stability: MDI regularization,
a inverse system matrix filter, and Laplacian power artifical viscosity.  One
or more approach may be used at the same time.

usage: run heatCircleLocalEws.py
          
"""

import matplotlib.pyplot as plt
from numpy import arange
from gax import *
import rbfCenters as rc
import rbfMisc as rm
import scipy.sparse as spr
import rbfLocal as LR

import warnings
warnings.filterwarnings('ignore')
plt.close('all')

phi = gax()

MDI = True   
FILTER = False
LAP = True
SAFE = False   # LU or Cholesky

mu, nuG, nuL = 0, 0, 0
order = 2   # Laplacian order

def LO(r,x,y,c,phi): return phi.L(r,c)

# --------------- centers -------------------------------------

N, Nb = 6000, 350
bpi = arange(Nb)
xi, yi = rc.circleCenters(N,cluster=False,ch=3,R=1,plt=False)
x, y = rc.circleBndCenters(xi,yi,1,Nb,tol=0.0125,plt=True)
x += 1
y += 1

print('boundary centers: {:d}'.format(Nb))
print('   total centers: {:d}'.format(len(x)))

# NOTE: both filter and lap individually work to shift the ews
if MDI:           # MDI regularization parameter
	mu = 5e-15           
if FILTER:        # hyperviscosity - inverse system matrix filter
	nuG = 1e-10      
if LAP:           # hyperviscosity - Laplacian power
	nuL = 1e-13


print('            MDI mu: {:1.2e}'.format(mu))
print(' inverse system nu: {:1.2e}'.format(nuG))
print('Laplacian power nu: {:1.2e}'.format(nuL))

# ------------ local setup -----------------------

ns = 25
X = list( zip(x,y) )
st = LR.stencils(X,ns)  

s0 = 2.4
ds = 0.01
s = LR.shapes(X,st,phi,0,s0,ds,verbose=False)
D = LR.weights(X,st,phi,s,LO,mu,nuG,nuL,2,order)
D[:Nb,:] = 0

# ---------------------------------------------------------------------

# rm.dmEWs(D,dt=0.0001,p1=True,p2=True)	 # also plots RK4 stability region
rm.dmEWs(D,dt=1,p1=True,p2=False)	
plt.show()

