"""
Solves a 2d diffusion equation with a forcing term

u_t = u_{xx} + u_{yy} + 4k^2(x^2 + y^2) \cos(k \left[ t - x^2 -y^2 \right]) - 5 k \sin(k \left[t - x^2 -y^2 \right])

in a circular domain with a MODIFIED partition of unity RBF method.  The initial conditions
and Dirichlet boundary conditions are taken from the exact solution

u(x,y,t) = 1 + \cos(k \left[t - x^2 - y^2 \right])

usage: run heatCircleSarraPu.py

see also: heatCirclePuSarraEws.py which calculates the eigenvalues of the
          discretized equation with zero Dirichlet BCs applied.
          
"""

 
import matplotlib.pyplot as plt
from numpy import arange, zeros_like, inf, cos, sin, eye
from numpy.linalg import norm
import rbfPU as PU
import matplotlib.tri as tri
import AnimatePlot as ap
import rbfCenters as rc
import rbfMisc as rm

from rbfClassX import rbfx
from gax import *

import warnings
warnings.filterwarnings('ignore')
plt.close('all')

phi = gax()

MDI = True   
FILTER = False
LAP = False
SAFE = False   # LU or Cholesky

mu, nuG, nuL = 0, 0, 0
order = 2   # Laplacian order


# --------------- centers -------------------------------------

N, Nb = 6000, 350
bpi = arange(Nb)
xi, yi = rc.circleCenters(N,cluster=False,ch=3,R=1,plt=False)
x, y = rc.circleBndCenters(xi,yi,1,Nb,tol=0.0125,plt=True)
N = len(x)
x += 1
y += 1

triang = tri.Triangulation(x, y)
t, dt, finalT = 0, 0.00004, 0.5
showPlot = 1000

print('boundary centers: {:d}'.format(Nb))
print('   total centers: {:d}'.format(len(x)))

if MDI:           # MDI regularization parameter
	mu = 5e-15           
if FILTER:        # hyperviscosity - inverse system matrix filter
	nuG = 8e-12     
if LAP:           # hyperviscosity - Laplacian power
	nuL = 7e-14


print('            MDI mu: {:1.2e}'.format(mu))
print(' inverse system nu: {:1.2e}'.format(nuG)) 
print('Laplacian power nu: {:1.2e}'.format(nuL))


# ---------- standard PU setup ----------------------------------

radius, xc, yc, st = PU.cover2d(x,y,5,0.75,0)

#PU.plotCover(x,y,xc,yc,radius)

X = list( zip(x,y) )
shape = PU.patchShape(X,st,phi,mu=0,s0=0.5,ds=0.01)

from W32 import *
phiC = W32()

n = len(st)                                     # number of patches
w = PU.patchWeights(st,xc,yc,x,y,radius,phiC)

d = []
for k in range(n):
	st0 = st[k]
	s = shape[k]
	r, rx, ry = rbfx.distanceMatrix2d(x[st0],y[st0])
	B, H = phi.rbf(r,s), phi.L(r,s)
	Hv = phi.hyperViscosity(2,order,s,r)
	n0 = B.shape[0]
	I = eye(n0)
	d.append( phi.dm( B, H - nuG*I - nuL*Hv, mu, False ) )
	
D = PU.globalDMSarra(N,st,w,d)
D[bpi,:] = 0

# ----------------------------------------------------------------

K = 2
def exactSolution(x,y,t): return 1.0 + cos(K*t - K*x**2 - K*y**2)

U = exactSolution(x,y,t)
u = zeros_like(U)

def F(v,t):
    v[bpi] = exactSolution(x[bpi],y[bpi],t)
    return D@v + 4*K**2*(x**2 + y**2)*cos(K*t - K*x**2 - K*y**2) - 5*K*sin(K*t - K*x**2 - K*y**2)
    
    
def makePlot(): 
	plt.tricontour(x, y, u, 15, linewidths=1.5, colors='k')
	plt.tricontourf(x, y, u, 15, cmap=plt.cm.jet)


n = 0
while t<finalT:
    u = rm.rk4(U,t,dt,F)
    u[bpi] = exactSolution(x[bpi],y[bpi],t)
    if n % showPlot == 0:
        ap.animate(makePlot)
    t += dt
    n += 1
    U = u.copy()

plt.colorbar()
exact = exactSolution(x,y,t)
print('max error: {:1.2e}'.format(norm( u-exact,inf)/max(abs(u))))
