import scipy as sp
import scipy.linalg as la
from abc import ABC, abstractmethod



class rbfc(ABC):
	"""
	The base class for compactly supoorted Wendland function RBFs.
	
	Compactly supported Wendland functions phi(s=3,k) which are
	strictly positive definite in R^d for d<=3 with smoothness 2*k

    subclasses:
	   W32 with C4 smoothness
       W33 with C6 smoothness
	
	usage: from rbfCompactClass import rbfc
	
	"""

#----------------------------------------------------------------------------
#----------------------- Abstract methods------------------------------------
#            used to define a common interface for all subclasses
#                   must be implemented by all subclasses.
#----------------------------------------------------------------------------

# ------------------------------------------------------------------------------		
# --------------------- rbf definition -----------------------------------------
# ------------------------------------------------------------------------------
		
	@abstractmethod
	def rbf(self,r,s): pass
	
# ------------------------------------------------------------------------------	
# ---------------------- derivatives -------------------------------------------
# ------------------------------------------------------------------------------

	@abstractmethod
	def D1(self,r,s,x):
		"""
		First partial derivative of the basis function wrt x in any space dimension.
		
		inputs:
			r    distance matrix
			s    parameter for compact support
			x    distance matrix of the variable that the derivative is with respect to,
				 for example rx, ry, rz, etc.
		output:
			d    first derivative of the basis function wrt to x
		
		"""
		pass
		
	@abstractmethod
	def D2(self,r,s,x):
		"""
		Second partial derivative of the basis function wrt x in any space dimension.
		
		inputs:
			r    distance matrix
			s    parameter for compact support
			x    distance matrix of the variable that the derivative is with respect to,
				 for example rx, ry, rz, etc.
		output:
			d    first derivative of the basis function wrt to x
		
		"""
	
# ------------------------------------------------------------------------------

	def pPlus(self, r, p): return (1-r)**p*(r<1)

# ------------------------------------------------------------------------------

	derivativeOrder = {1:D1, 2:D2}

	def D(self, r, s, x, dn):
		return self.derivativeOrder[dn](self, r, s, x)
